#!/bin/bash
SCRIPT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" >/dev/null 2>&1 && pwd)"

cwd=$(pwd)

PROJECT_DIR_NAME=${cwd##*/}

# Load colors and the I/O functions
source ${SCRIPT_DIR}/utils/io.sh

source ${SCRIPT_DIR}/functions.sh

function usage() {
  echo "${YELLOW}Description: ${RESET}"
  echo "    StarterKit Upgrader. "
  echo ""
  echo "${YELLOW}Usage: ${RESET}"
  echo "    ./bin/upgrade.sh [options]"
  echo ""
  echo "${YELLOW}Options: ${RESET}"
  echo "   ${GREEN} -h, --help     ${RESET} Display this help message"
  echo "   ${GREEN} -v, --version  ${RESET} The version of StarterKit to upgrade to"
}

function moveFilesToTemp() {
  local __path=${1}

  if [[ -z ${__path} ]]; then
    echo "Missing path"
    exit 1
  fi

  if [[ -d "${cwd}/${__path}" ]]; then
    rsync -a "${cwd}/${__path}/" "${TEMP_DIR}/${__path}"
  elif [[ -f "${cwd}/${__path}" ]]; then
    cp "${cwd}/${__path}" "${TEMP_DIR}/${__path}"
  fi
}

function mergeComposerJson() {
  local __tempFile=${1}
  local __currentFile=${2}

  if [[ -z ${__currentFile} ]]; then
    echo "Missing composer.json"
    exit 1
  fi

  echo $(node -e "
  const fs = require('fs');
  const tj = require('${__tempFile}');
  let cj = require('${__currentFile}');

  cj.require = {...cj.require,...tj.require}
  cj.scripts = {...cj.scripts,...tj.scripts}
  cj.autoload['psr-4'] = {...cj.autoload['psr-4'],...tj.autoload['psr-4']}

  fs.writeFile('${__tempFile}', JSON.stringify(cj,null,4), (err) => err && console.log(err));
  ")
  echo "" >> ${__tempFile}
}

# Read arguments
POSITIONAL=()
while [[ $# -gt 0 ]]; do
  key="$1"
  case $key in
  -h | --help)
    usage
    exit 0
    shift # past argument
    ;;
  -v | --version)
    VERSION=${2}
    shift # past argument
    shift # past argument
    ;;
  *) # unknown option
    POSITIONAL+=("$1") # save it in an array for later
    shift              # past argument
    ;;
  esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters
# Done reading arguments

if [[ -z ${VERSION} ]]; then
  ioInfo "Trying to find the latest version..."
  VERSIONS=($(getStarterKitVersions))

  if [[ $? != 0 ]] || [[ -z ${VERSIONS} ]]; then
    ioError "Failed to automatically fetch latest StarterKit version."
    ioNote "You can find the available versions on \"https://composer.infomaker.io/#!/everyware-starter-kit\"." \
      "Then try to run upgrade again with ' --version <version>'"
    exit 1
  fi

  VERSION=${VERSIONS[0]}
fi

ioInfo "Prepare upgrade of Starter Kit..."

STARTER_KIT_ZIP="https://composer.infomaker.io/dist/infomaker-everyware-starter-kit-${VERSION}.zip"
TEMP_DIR="${cwd}/tmp/stkt-${VERSION}"

if [[ ! -d ${TEMP_DIR} ]]; then
  ioInfo "Downloading Starter Kit v${VERSION}"
  curl -L -o "${TEMP_DIR}.zip" --create-dirs ${STARTER_KIT_ZIP}
  unzip -q "${TEMP_DIR}.zip" -d ${TEMP_DIR}
fi

cd ${TEMP_DIR}

# Create a .env.example file
[[ ! -f .project-env ]] || [[ -f .env.example ]] || mv .project-env .env.example

# Create project .gitignore
[[ ! -f .project-gitignore ]] || [[ -f .gitignore ]] || mv .project-gitignore .gitignore

REMOVE_FILES=(
  mysql
  .editorconfig
  .eslintrc.js
  .sass-lint.yml
  gulpfile.js
  gulp-config.json
  package.json
  CHANGELOG.md
)

rm -rf ${REMOVE_FILES[@]}

cd ${cwd}

ioInfo "Secure files that should be unchanged"

moveFilesToTemp "lua/utils"
moveFilesToTemp "nginx/app"
moveFilesToTemp "www/wp-config/wp-salt.php"

mergeComposerJson ${TEMP_DIR}/composer.json ${cwd}/composer.json

ioSuccess "Preparations done!!"

# Compare bin-scripts to determine if some of them should be removed
ioInfo "Check for unknown files in ${SCRIPT_DIR}"

# Make sure that the new version has an upgrade script
[[ -f ${TEMP_DIR}/bin/upgrade.sh ]] || cp ${SCRIPT_DIR}/upgrade.sh ${TEMP_DIR}/bin/upgrade.sh

STKT_SCRIPTS=($(ls ${TEMP_DIR}/bin))

for file in "${SCRIPT_DIR}"/*; do
  filename=${file##*/}

  # If script-file dose not exists in new StarterKit version we offer to remove it
  if [[ ! " ${STKT_SCRIPTS[@]} " =~ " ${filename} " ]]; then
    echo ""
    echo "\"${filename}\" is not part of the new StarterKit version."
    read -p "Do you wish to remove it? [y/N] " -n 1 -r
    echo ""
    if [[ $REPLY =~ ^[Yy]$ ]]; then
      echo "Removing ${SCRIPT_DIR}/${filename}"
      rm -rf ${SCRIPT_DIR}/${filename}
    fi
  fi
done

# Rename new upgrade file if it will change
if [[ $(diff ${TEMP_DIR}/bin/upgrade.sh ${SCRIPT_DIR}/upgrade.sh) ]]; then
  mv ${TEMP_DIR}/bin/upgrade.sh ${TEMP_DIR}/bin/upgrade-${VERSION}.sh
fi

# Make sure that bin scripts are still executable
[[ ! -d ${TEMP_DIR}/bin ]] || chmod -R 755 ${TEMP_DIR}/bin

# Make sure that .platform scripts are still executable
[[ ! -d ${TEMP_DIR}/.platform ]] || chmod -R 755 ${TEMP_DIR}/.platform

# Make sure that the aws deploy script is still executable
[[ ! -f ${TEMP_DIR}/aws-deploy.sh ]] || chmod 755 ${TEMP_DIR}/aws-deploy.sh

ioInfo "Proceed to update the project code..."

REMOVE_FILES=(
  .ebextensions
  .platform
  Dockerrun.aws.json
  lua
  rockspec
  www/wp-config
  www/wp-content/themes/default-theme
  www/wp-content/themes/placeholder-theme
)

rm -rf ${REMOVE_FILES[@]}

rsync -a ${TEMP_DIR}/ .

# Clean Up
rm -rf ./tmp*

ioSuccess "All done!! You have successfully updated your project to Starter Kit v${VERSION}"

ioNote "Make sure to review all changes before committing them." \
  "Some of the files that have been updated needs to be manually managed to match the projects configuration."

# Check if any of the EB-config files has changed
git diff --exit-code ${cwd}/.ebextensions/ >>/dev/null
if [[ ${?} == 1 ]]; then

  ioNote "Elastic Beanstalk config has changes with the new version of StarterKit. " \
    "Further changes might have to be done in AWS in order for these changes to work. "

  git diff --exit-code ${cwd}/.ebextensions/
fi

if [[ -f ${SCRIPT_DIR}/upgrade-${VERSION}.sh ]]; then

  ioNote "It seems the upgrade script has been changed with the new version of StarterKit. " \
    "You need to manually update it. To do this run the following command: " \
    "" \
    "mv bin/upgrade-${VERSION}.sh bin/upgrade.sh "
fi
